@@ -85,9 +85,7 @@ class RedshiftTracker:
     def unpatch_method(obj, original_attr, patched_attr="__dbnd_patched__"):
         method = getattr(obj, original_attr)
         if hasattr(method, patched_attr):
-            setattr(
-                obj, original_attr, getattr(method, patched_attr),
-            )
+            setattr(obj, original_attr, getattr(method, patched_attr))
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.unpatch_method(DbndCursorWrapper, "execute")
@@ -103,7 +101,7 @@ class RedshiftTracker:
         # we clean all the batch of operations we reported so we don't report twice
         self.operations = []
 
-    def set_read_dataframe(self, dataframe):
+    def set_dataframe(self, dataframe):
         """
         set dataframe
         Args:
@@ -116,7 +114,7 @@ class RedshiftTracker:
                 self.dataframe = dataframe
             else:
                 logger.exception(
-                    "Error occurred during set dataframe. provided dataframe is not valid",
+                    "Error occurred during set dataframe. provided dataframe is not valid"
                 )
         except Exception as e:
             logger.exception("Error occurred during set dataframe: %s", self.dataframe)
@@ -159,26 +157,16 @@ class RedshiftTracker:
         # looks for tables schemas
         tables = chain.from_iterable(op.tables for op in operations if not op.is_file)
         # get df schema if exist
-        if self.dataframe is not None:
-            try:
-                df_schema = self.dataframe.dtypes.to_dict()
-                df_schema = ((k, str(v)) for k, v in df_schema.items())
-            except Exception as e:
-                df_schema = None
-                logger.exception(
-                    "Error occurred during build schema from dataframe: %s",
-                    self.dataframe,
-                )
-                log_exception_to_server(e)
-        else:
-            df_schema = None
+        df_schema = build_schema_from_dataframe(self.dataframe)
 
         tables_schemas: Dict[str, DTypes] = {}
-
         for table in tables:
-            table_schema = get_redshift_table_schema(connection, table)
-            if table_schema:
-                tables_schemas[table] = table_schema
+            if df_schema:
+                tables_schemas[table] = df_schema
+            else:
+                table_schema = get_redshift_table_schema(connection, table)
+                if table_schema:
+                    tables_schemas[table] = table_schema
 
         operations: List[SqlOperation] = [
             op.evolve_schema(tables_schemas, df_schema) for op in operations
@@ -201,6 +189,22 @@ class RedshiftTracker:
             )
 
 
+def build_schema_from_dataframe(dataframe):
+    if dataframe is not None:
+        try:
+            df_schema = dataframe.dtypes.to_dict()
+            df_schema = dict((k, str(v)) for k, v in df_schema.items())
+        except Exception as e:
+            df_schema = None
+            logger.exception(
+                "Error occurred during build schema from dataframe: %s", dataframe
+            )
+            log_exception_to_server(e)
+    else:
+        df_schema = None
+    return df_schema
+
+
 def get_redshift_table_schema(connection, table) -> Optional[DTypes]:
     desc_results = redshift_query(
         connection,
@@ -272,6 +276,7 @@ def build_redshift_operations(
         query_id=None,
         success=success,
         error=error,
+        dataframe=dataframe,
     )
 
     if READ in extracted and WRITE not in extracted:
@@ -279,10 +284,7 @@ def build_redshift_operations(
         # description contains the schema of the operation.
         schema = None
         read = build_operation(
-            extracted_schema=extracted[READ],
-            dtypes=schema,
-            op_type=READ,
-            dataframe=dataframe,
+            extracted_schema=extracted[READ], dtypes=schema, op_type=READ
         )
         operations.append(read)
     else:
@@ -290,10 +292,7 @@ def build_redshift_operations(
         # schema.
         if READ in extracted:
             read = build_operation(
-                extracted_schema=extracted[READ],
-                dtypes=None,
-                op_type=READ,
-                dataframe=dataframe,
+                extracted_schema=extracted[READ], dtypes=None, op_type=READ
             )
             operations.append(read)
 
